package net.wanji.web.task;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.feign.pojo.result.JsonViewObject;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.dto.SignalStatusLogDTO;
import net.wanji.web.entity.TCrossControlHist;
import net.wanji.web.entity.TDeviceStatusInfo;
import net.wanji.web.entity.TDeviceStatusLog;
import net.wanji.web.mapper.ControlHistMapper;
import net.wanji.web.mapper.TDeviceStatusLogMapper;
import net.wanji.web.mapper.TDeviceStatusMapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2023/01/03 10:43
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class SignalStatusTask {
    private final UtcFeignClients utcFeignClients;
    private final TDeviceStatusMapper tDeviceStatusMapper;
    private final ControlHistMapper controlHistMapper;
    private final TDeviceStatusLogMapper tDeviceStatusLogMapper;

    @Scheduled(fixedRate = 1 * 60 * 1000)
    @Transactional
    public void syncSignalStatus() {
        try {
            JsonViewObject jsonViewObject = utcFeignClients.runningStatusAlarm();
            if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
                log.error("定时任务同步信号机设备状态utcService调用异常");
                return;
            }
            List<SignalStatusLogDTO> content = (List<SignalStatusLogDTO>) jsonViewObject.getContent();
            content.forEach(signalStatusLogDTO -> {
                String signalId = signalStatusLogDTO.getSignalId();
                Integer currentSignalStatus = signalStatusLogDTO.getStatus() == null ? 0 : signalStatusLogDTO.getStatus();
                Integer currentFaultType = signalStatusLogDTO.getFaultType() == null ? 0 : signalStatusLogDTO.getFaultType();

                // 更新数据库状态
                LambdaQueryWrapper<TDeviceStatusInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                if (StringUtils.isNotEmpty(signalId)) {
                    lambdaQueryWrapper.eq(TDeviceStatusInfo::getCode, signalId);
                    TDeviceStatusInfo tDeviceStatusInfo = tDeviceStatusMapper.selectOne(lambdaQueryWrapper);
                    if (isExucuteUpdate(currentSignalStatus, currentFaultType, tDeviceStatusInfo)) {
                        tDeviceStatusInfo.setStatus(currentSignalStatus);
                        tDeviceStatusInfo.setFaultType(currentFaultType);
                        tDeviceStatusMapper.updateById(tDeviceStatusInfo);
                    }
                    insertDeviceStatusLog(signalStatusLogDTO, signalId, currentSignalStatus, currentFaultType);
                }
                insertControlHist(signalStatusLogDTO);
            });
        } catch (Exception e) {
            log.error("定时任务同步信号机设备状态远程utcService调用异常", e);
        }
    }

    /**
     * 插入控制历史表
     *
     * @param signalStatusLogDTO
     */
    private void insertControlHist(SignalStatusLogDTO signalStatusLogDTO) {
        // 更新控制历史表
        String crossId = signalStatusLogDTO.getCrossId();
        Integer controlType = signalStatusLogDTO.getControlType();
        TCrossControlHist tCrossControlHist = controlHistMapper.selectRecentOne(crossId);
        if (Objects.nonNull(tCrossControlHist) && !Objects.equals(controlType, tCrossControlHist.getType())) {
            controlHistMapper.insertOne(crossId, controlType, "未知");
        }
    }

    /**
     * 信号机状态插入设备日志表
     *
     * @param signalStatusLogDTO
     * @param signalId
     * @param currentSignalStatus
     * @param currentFaultType
     */
    private void insertDeviceStatusLog(SignalStatusLogDTO signalStatusLogDTO, String signalId, Integer currentSignalStatus, Integer currentFaultType) {
        // 插入日志表
        TDeviceStatusLog tDeviceStatusLogInfoDB = tDeviceStatusLogMapper.selectLastOne(signalId);
        // DB数据是故障，如果当前正常非故障，状态为  FaultType=100 故障已处理
        if (Objects.nonNull(tDeviceStatusLogInfoDB) || tDeviceStatusLogInfoDB.getFaultType() != 0 && currentFaultType == 0) {
            tDeviceStatusLogInfoDB.setStatus(currentSignalStatus);
            tDeviceStatusLogInfoDB.setFaultType(100);
            tDeviceStatusLogMapper.insert(tDeviceStatusLogInfoDB);
        } else {
            TDeviceStatusLog tDeviceStatusLog = new TDeviceStatusLog();
            tDeviceStatusLog.setCode(signalId);
            tDeviceStatusLog.setName(signalStatusLogDTO.getSignalId());
            tDeviceStatusLog.setType(1);
            tDeviceStatusLog.setStatus(signalStatusLogDTO.getStatus());
            tDeviceStatusLog.setFaultType(currentFaultType);
            tDeviceStatusLogMapper.insert(tDeviceStatusLog);
        }
    }

    private static boolean isExucuteUpdate(Integer currentSignalStatus, Integer currentFaultType, TDeviceStatusInfo tDeviceStatusInfo) {
        return Objects.nonNull(tDeviceStatusInfo) && (currentSignalStatus != tDeviceStatusInfo.getStatus() || currentFaultType != tDeviceStatusInfo.getFaultType());
    }
}
